from django.contrib.sites.managers import CurrentSiteManager
from edc_model.models import HistoricalRecords
from edc_visit_tracking.managers import CrfModelManager

from .crf_no_manager_model_mixin import CrfNoManagerModelMixin


class CrfModelMixin(CrfNoManagerModelMixin):

    on_site = CurrentSiteManager()
    objects = CrfModelManager()
    history = HistoricalRecords(inherit=True)

    class Meta(CrfNoManagerModelMixin.Meta):
        abstract = True



from django.contrib.sites.managers import CurrentSiteManager
from edc_action_item.models import ActionNoManagersModelMixin
from edc_identifier.model_mixins import TrackingModelMixin
from edc_model.models.historical_records import HistoricalRecords
from edc_visit_tracking.managers import CrfModelManager

from .crf_model_mixin import CrfNoManagerModelMixin


class CrfWithActionModelMixin(
    CrfNoManagerModelMixin,
    ActionNoManagersModelMixin,
    TrackingModelMixin,
):

    action_name = None
    tracking_identifier_prefix = ""

    on_site = CurrentSiteManager()
    objects = CrfModelManager()
    history = HistoricalRecords(inherit=True)

    class Meta(CrfNoManagerModelMixin.Meta):
        abstract = True



from datetime import datetime
from typing import Optional

from edc_consent.utils import get_consent_model_cls
from edc_screening.utils import get_subject_screening_model_cls
from edc_utils import age, to_utc
from edc_visit_tracking.stubs import SubjectVisitModelStub


class PrnFormValidatorMixin:
    """A mixin of common properties needed for CRF validation
    to be declared with FormValidator.

    Assumes model has a key to subject_visit
    """

    @property
    def subject_identifier(self) -> str:
        return self.cleaned_data.get("subject_identifier")

    @property
    def report_datetime(self) -> datetime:
        try:
            return self.cleaned_data.get("report_datetime")
        except AttributeError:
            return self.subject_visit.report_datetime

    @property
    def subject_screening(self):
        return get_subject_screening_model_cls().objects.get(
            subject_identifier=self.subject_identifier
        )

    @property
    def subject_consent(self):
        return get_consent_model_cls().objects.get(subject_identifier=self.subject_identifier)

    @property
    def age_in_years(self) -> int:
        return age(self.subject_consent.dob, to_utc(self.report_datetime)).years


class CrfFormValidatorMixin(PrnFormValidatorMixin):
    @property
    def subject_visit(self) -> Optional[SubjectVisitModelStub]:
        """Returns a subject visit model instance or None"""
        try:
            subject_visit = self.instance.subject_visit
        except AttributeError:
            subject_visit = self.cleaned_data.get("subject_visit")
        return subject_visit

    @property
    def subject_identifier(self) -> str:
        return self.subject_visit.subject_identifier



from django.db import models
from edc_constants.choices import DOCUMENT_STATUS

from . import crf_status_default
from .update_crf_status_for_instance import update_crf_status_for_instance


class CrfStatusModelMixin(models.Model):
    crf_status = models.CharField(
        verbose_name="CRF status",
        max_length=25,
        choices=DOCUMENT_STATUS,
        default=crf_status_default,
        help_text="If some data is still pending, flag this CRF as incomplete",
    )

    crf_status_comments = models.TextField(
        verbose_name="Any comments related to status of this CRF",
        null=True,
        blank=True,
        help_text="for example, why some data is still pending",
    )

    def update_crf_status_for_instance(self):
        return update_crf_status_for_instance(self)

    class Meta:
        abstract = True



# Generated by Django 3.0.4 on 2020-05-12 02:04

import _socket
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CrfStatus",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "visit_schedule_name",
                    models.CharField(
                        editable=False,
                        help_text='the name of the visit schedule used to find the "schedule"',
                        max_length=25,
                    ),
                ),
                ("schedule_name", models.CharField(editable=False, max_length=25)),
                (
                    "visit_code",
                    models.CharField(editable=False, max_length=25, null=True),
                ),
                (
                    "visit_code_sequence",
                    models.IntegerField(
                        blank=True,
                        default=0,
                        help_text="An integer to represent the sequence of additional appointments relative to the base appointment, 0, needed to complete data collection for the timepoint. (NNNN.0)",
                        null=True,
                        verbose_name="Sequence",
                    ),
                ),
                ("label_lower", models.CharField(max_length=150, null=True)),
            ],
            options={
                "verbose_name": "CRF Status",
                "verbose_name_plural": "CRF Status",
            },
        ),
    ]



import abc
from datetime import datetime

from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.db.models import ForeignKey, OneToOneField, options
from edc_sites.models import SiteModelMixin
from edc_visit_tracking.model_mixins.crfs import InlineVisitMethodsModelMixin
from edc_visit_tracking.stubs import SubjectVisitModelStub, TSubjectVisitModelStub

options.DEFAULT_NAMES = options.DEFAULT_NAMES + ("crf_inline_parent",)


class CrfInlineModelMixin(InlineVisitMethodsModelMixin, SiteModelMixin, models.Model):
    """A mixin for models used as inlines in ModelAdmin."""

    def __init__(self, *args, **kwargs) -> None:
        """Try to detect the inline parent model attribute
        name or raise.
        """
        super().__init__(*args, **kwargs)
        try:
            self._meta.crf_inline_parent
        except AttributeError:
            fks = [
                field
                for field in self._meta.fields
                if isinstance(field, (OneToOneField, ForeignKey))
            ]
            if len(fks) == 1:
                self.__class__._meta.crf_inline_parent = fks[0].name
            else:
                raise ImproperlyConfigured(
                    "CrfInlineModelMixin cannot determine the "
                    "inline parent model name. Got more than one foreign key. "
                    "Try declaring \"crf_inline_parent = '<field name>'\" "
                    "explicitly in Meta."
                )

    def __str__(self) -> str:
        return str(self.parent_instance.subject_visit)

    @abc.abstractmethod
    def natural_key(self) -> tuple:
        return tuple()

    @property
    def parent_instance(self):
        """Return the instance of the inline parent model."""
        return getattr(self, self._meta.crf_inline_parent)

    @property
    def parent_model(self) -> TSubjectVisitModelStub:
        """Return the class of the inline parent model."""
        field = getattr(self.__class__, self._meta.crf_inline_parent).field
        try:
            return field.rel.to
        except AttributeError:
            return field.remote_field.model  # django 2.0 +

    @property
    def subject_visit(self) -> SubjectVisitModelStub:
        """Return the instance of the inline parent model's visit
        model.
        """
        return getattr(self.parent_instance, self.visit_model_attr())

    def visit_model_attr(self):
        return self.parent_model.visit_model_attr()

    @property
    def report_datetime(self) -> datetime:
        """Return the instance of the inline parent model's
        report_datetime.
        """
        return self.subject_visit.report_datetime

    class Meta:
        crf_inline_parent: str = None
        abstract = True



from datetime import datetime
from typing import Any

from edc_appointment.form_validators import WindowPeriodFormValidatorMixin
from edc_consent.form_validators import ConsentFormValidatorMixin
from edc_form_validators import FormValidator
from edc_registration.models import RegisteredSubject
from edc_utils import formatted_datetime


class CrfFormValidator(
    WindowPeriodFormValidatorMixin, ConsentFormValidatorMixin, FormValidator
):
    """Form validator for CRfs.

    CRFs have a FK to subject_visit.
    """

    def _clean(self) -> None:
        self.validate_crf_report_datetime()
        super()._clean()

    def validate_crf_report_datetime(self):
        if self.cleaned_data.get("report_datetime"):
            # falls within appointment's window period
            self.validate_crf_datetime_in_window_period(
                self.appointment,
                self.cleaned_data.get("report_datetime"),
                "report_datetime",
            )
            # falls within a valid consent period
            self.get_consent_for_period_or_raise(
                self.cleaned_data.get("report_datetime"), form_field="report_datetime"
            )
            # not before consent date
            if self.cleaned_data.get("report_datetime") < self.consent_datetime:
                self.raise_validation_error(
                    {
                        "report_datetime": (
                            "Invalid. Cannot be before date of consent. "
                            "Participant consent on "
                            f"{formatted_datetime(self.consent_datetime)}"
                        )
                    }
                )

    @property
    def appointment(self) -> Any:
        return self.subject_visit.appointment

    @property
    def subject_visit(self) -> Any:
        """Returns a subject visit model instance or None"""
        try:
            subject_visit = self.instance.subject_visit
        except AttributeError:
            subject_visit = self.cleaned_data.get("subject_visit")
        return subject_visit

    @property
    def subject_identifier(self) -> str:
        return self.appointment.subject_identifier

    @property
    def consent_datetime(self) -> datetime:
        return RegisteredSubject.objects.get(
            subject_identifier=self.subject_identifier
        ).consent_datetime



from django import forms
from django.conf import settings
from edc_consent.modelform_mixins import RequiresConsentModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import SubjectScheduleModelFormMixin
from edc_visit_tracking.modelform_mixins import VisitTrackingModelFormMixin


class CrfModelFormMixin(
    SiteModelFormMixin,
    RequiresConsentModelFormMixin,
    SubjectScheduleModelFormMixin,
    VisitTrackingModelFormMixin,
    FormValidatorMixin,
):
    pass


class InlineCrfModelFormMixin(FormValidatorMixin, forms.ModelForm):
    visit_model = settings.SUBJECT_VISIT_MODEL



from django.conf import settings
from django.db import models
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow

from edc_crf.crf_model_mixin import CrfModelMixin
from edc_crf.crf_status_model_mixin import CrfStatusModelMixin


class Crf(CrfModelMixin, CrfStatusModelMixin, BaseUuidModel):

    subject_visit = models.ForeignKey(settings.SUBJECT_VISIT_MODEL, on_delete=models.PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    f1 = models.CharField(max_length=50, null=True, blank=True)

    f2 = models.CharField(max_length=50, null=True, blank=True)

    f3 = models.CharField(max_length=50, null=True, blank=True)

    class Meta(BaseUuidModel.Meta):
        pass


class Prn(BaseUuidModel):

    subject_identifier = models.CharField(max_length=50, null=True, blank=True)

    report_datetime = models.DateTimeField(default=get_utcnow)

    f1 = models.CharField(max_length=50, null=True, blank=True)

    f2 = models.CharField(max_length=50, null=True, blank=True)

    f3 = models.CharField(max_length=50, null=True, blank=True)

    class Meta(BaseUuidModel.Meta):
        pass



from django.db import models
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model import models as edc_models
from edc_visit_schedule.model_mixins import (
    VisitCodeFieldsModelMixin,
    VisitScheduleFieldsModelMixin,
)


class CrfStatus(
    NonUniqueSubjectIdentifierFieldMixin,
    VisitScheduleFieldsModelMixin,
    VisitCodeFieldsModelMixin,
    edc_models.BaseUuidModel,
):
    label_lower = models.CharField(max_length=150, null=True)

    class Meta(edc_models.BaseUuidModel.Meta):
        verbose_name = "CRF Status"
        verbose_name_plural = "CRF Status"
